/*
 * Copyright (c) 2016, 2017, 2018, 2019 FabricMC
 * Copyright 2022 The Quilt Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.fabricmc.fabric.api.item.v1;

import net.minecraft.class_1792;
import net.minecraft.class_1799;
import net.minecraft.class_4174;
import org.jetbrains.annotations.Nullable;
import org.quiltmc.qsl.item.setting.api.RecipeRemainderLogicHandler;

/*
 * Fabric-provided extensions for {@link ItemStack}.
 * This interface is automatically implemented on all item stacks via Mixin and interface injection.
 */
public interface FabricItemStack {
	/**
	 * Return a leftover item for use in recipes.
	 *
	 * <p>See {@link FabricItem#getRecipeRemainder(class_1799)} for a more in depth description.
	 *
	 * <p>Stack-aware version of {@link class_1792#method_7858()}.
	 *
	 * @return the leftover item
	 */
	default class_1799 getRecipeRemainder() {
		return RecipeRemainderLogicHandler.getRemainder((class_1799) (Object) this, null);
	}

	/**
	 * Stack-aware version of {@link class_1792#method_19264()}.
	 * See {@link FabricItem#getFoodComponent(class_1799)} for a more in depth description.
	 *
	 * @return this item stack's {@link class_4174}, or {@code null} if none was set
	 */
	default @Nullable class_4174 getFoodComponent() {
		return ((class_1799) this).method_7909().method_19264(((class_1799) this));
	}
}
